# Adapte de ChannelLogger - Beuc 2020-12
# Logs récents, stateless
###
# Copyright (c) 2002-2004, Jeremiah Fincher
# Copyright (c) 2009-2010, James McCoy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

import os
import sys
import time

import supybot.conf as conf
import supybot.world as world
import supybot.ircdb as ircdb
import supybot.irclib as irclib
import supybot.utils.minisix as minisix
import supybot.ircmsgs as ircmsgs
import supybot.ircutils as ircutils
import supybot.registry as registry
import supybot.callbacks as callbacks
from supybot.commands import wrap, optional
try:
    from supybot.i18n import PluginInternationalization
    _ = PluginInternationalization('LogsRecentsSansTrace')
except ImportError:
    # Placeholder that allows to run the plugin on a bot
    # without the i18n module
    _ = lambda x: x

if minisix.PY2:
    from io import open

class MemoireLog(object):
    def __init__(self, size):
        self.lines = []
        self.size = size
    def start(self, timestamp):
        self.lines.append({ 'timestamp': timestamp,
                            'message': ''})
        while len(self.lines) > self.size:
            del self.lines[0]
    def write(self, message):
        self.lines[-1]['message'] += message

class LogsRecentsSansTrace(callbacks.Plugin):
    """This plugin allows the bot to log RECENT channel conversations NOT to disk."""
    noIgnore = True
    def __init__(self, irc):
        self.__parent = super(LogsRecentsSansTrace, self)
        self.__parent.__init__(irc)
        self.logs = {}

    def reset(self):
        self.logs.clear()

    def _logs(self):
        for logs in self.logs.values():
            for log in logs.values():
                yield log

    def getLog(self, irc, channel):
        try:
            logs = self.logs[irc]
        except KeyError:
            logs = ircutils.IrcDict()
            self.logs[irc] = logs
        if channel in logs:
            return logs[channel]
        else:
            logs[channel] = MemoireLog(self.registryValue('maxLines', channel))
            return logs[channel]

    def timestamp(self, log):
        format = conf.supybot.log.timestampFormat()
        if format:
            string = time.strftime(format)
            if minisix.PY2:
                string = string.decode('utf8', 'ignore')
            log.start(string)

    def normalizeChannel(self, irc, channel):
        return ircutils.toLower(channel)

    def doLog(self, irc, channel, s, *args):
        if not self.registryValue('enable', channel):
            return
        s = format(s, *args)
        channel = self.normalizeChannel(irc, channel)
        log = self.getLog(irc, channel)
        if self.registryValue('timestamp', channel):
            self.timestamp(log)
        if self.registryValue('stripFormatting', channel):
            s = ircutils.stripFormatting(s)
        if minisix.PY2:
            s = s.decode('utf8', 'ignore')
        log.write(s)

    def doPrivmsg(self, irc, msg):
        (recipients, text) = msg.args
        for channel in recipients.split(','):
            if irc.isChannel(channel):
                noLogPrefix = self.registryValue('noLogPrefix', channel)
                cap = ircdb.makeChannelCapability(channel, 'logChannelMessages')
                try:
                    logChannelMessages = ircdb.checkCapability(msg.prefix, cap,
                        ignoreOwner=True)
                except KeyError:
                    logChannelMessages = True
                nick = msg.nick or irc.nick
                if msg.tagged('ChannelLogger__relayed'):
                    (nick, text) = text.split(' ', 1)
                    nick = nick[1:-1]
                    msg.args = (recipients, text)
                if (noLogPrefix and text.startswith(noLogPrefix)) or \
                        not logChannelMessages:
                    text = '-= THIS MESSAGE NOT LOGGED =-'
                if ircmsgs.isAction(msg):
                    self.doLog(irc, channel,
                               '* %s %s', nick, ircmsgs.unAction(msg))
                else:
                    self.doLog(irc, channel, '<%s> %s', nick, text)

    def doNotice(self, irc, msg):
        (recipients, text) = msg.args
        for channel in recipients.split(','):
            if irc.isChannel(channel):
                self.doLog(irc, channel, '-%s- %s', msg.nick, text)

    def doNick(self, irc, msg):
        oldNick = msg.nick
        newNick = msg.args[0]
        for channel in msg.tagged('channels'):
            self.doLog(irc, channel,
                       '*** %s is now known as %s', oldNick, newNick)

    def doInvite(self, irc, msg):
        (target, channel) = msg.args
        self.doLog(irc, channel,
                   '*** %s <%s> invited %s to %s',
                   msg.nick, msg.prefix, target, channel)

    def doJoin(self, irc, msg):
        for channel in msg.args[0].split(','):
            if(self.registryValue('showJoinParts', channel)):
                self.doLog(irc, channel,
                           '*** %s <%s> has joined %s',
                           msg.nick, msg.prefix, channel)

    def doKick(self, irc, msg):
        if len(msg.args) == 3:
            (channel, target, kickmsg) = msg.args
        else:
            (channel, target) = msg.args
            kickmsg = ''
        if kickmsg:
            self.doLog(irc, channel,
                       '*** %s was kicked by %s (%s)',
                       target, msg.nick, kickmsg)
        else:
            self.doLog(irc, channel,
                       '*** %s was kicked by %s', target, msg.nick)

    def doPart(self, irc, msg):
        if len(msg.args) > 1:
            reason = " (%s)" % msg.args[1]
        else:
            reason = ""
        for channel in msg.args[0].split(','):
            if(self.registryValue('showJoinParts', channel)):
                self.doLog(irc, channel,
                           '*** %s <%s> has left %s%s',
                           msg.nick, msg.prefix, channel, reason)

    def doMode(self, irc, msg):
        channel = msg.args[0]
        if irc.isChannel(channel) and msg.args[1:]:
            self.doLog(irc, channel,
                       '*** %s sets mode: %s %s',
                       msg.nick or msg.prefix, msg.args[1],
                        ' '.join(msg.args[2:]))

    def doTopic(self, irc, msg):
        if len(msg.args) == 1:
            return # It's an empty TOPIC just to get the current topic.
        channel = msg.args[0]
        self.doLog(irc, channel,
                   '*** %s changes topic to "%s"', msg.nick, msg.args[1])

    def doQuit(self, irc, msg):
        if len(msg.args) == 1:
            reason = " (%s)" % msg.args[0]
        else:
            reason = ""
        for channel in msg.tagged('channels'):
            if(self.registryValue('showJoinParts', channel)):
                self.doLog(irc, channel,
                           '*** %s <%s> has quit IRC%s',
                           msg.nick, msg.prefix, reason)

    def outFilter(self, irc, msg):
        # Gotta catch my own messages *somehow* :)
        # Let's try this little trick...
        if msg.command in ('PRIVMSG', 'NOTICE'):
            # Other messages should be sent back to us.
            m = ircmsgs.IrcMsg(msg=msg, prefix=irc.prefix)
            if msg.tagged('relayedMsg'):
                m.tag('ChannelLogger__relayed')
            self(irc, m)
        return msg

    """Sends the recent log to new users"""
    def logsrecents(self, irc, msg, args, channel, nombre_de_lignes):
        """[<channel>] [<nombre_de_lignes>]
    
        Renvoie les dernières lignes du chat sur le channel en cours,
        ou du channel spécifié (si vous y êtes connecté).
        """
        #irc.reply("[%s] [%s] [%s]" % (irc, type(msg), args))
        #channel = msg.args[0]
        if channel not in irc.state.channels:
            irc.error('Je ne connais pas ce salon.', Raise=True)
        if msg.nick not in irc.state.channels[channel].users:
            # pas d'espionnage du chan sans l'avoir rejoint
            irc.error('Vous devez être dans %s pour utiliser cette commande.' % channel, Raise=True)

        log = self.getLog(irc, channel)
        irc.reply("Je t'envoie (en privé) les dernières %d/%d lignes (max %d) de %s" %
                  (min(nombre_de_lignes, len(log.lines)), len(log.lines), log.size, channel))
        for l in log.lines[-nombre_de_lignes:]:
            irc.reply("%s: %s" % (l['timestamp'], l['message']),
                      private=True, to=msg.nick)
    logsrecents = wrap(logsrecents, ['channel', optional('int',20)])

Class = LogsRecentsSansTrace
# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
